;#######################################################
#NoEnv
#InstallKeybdHook
#InstallMouseHook 
SendMode Input
SetTitleMatchMode, RegEx

;/* Timing is very important in this game. Want our presses to be as fast as
; * possible, and held down just long enough for the game to register them */
SetKeyDelay, -1, 22
SetMouseDelay 1


;/*****************************************************************************
;* Autohotkey Script for Kingdoms of Amalur Reckoning - Dual Weapons v1.0
;*
;* - Primary purpose to to allow rebinding of the right mouse button, and
;*   optional additional side mouse buttons (up to 2).
;*
;* - Additional "Experimental" features: - Adding ability to have a dedicatetd
;*      input for "Secondary attack" like on the gamepad.
;*
;* - turning "select ability" into a "use ability" hotkey.
;*
;* - making a seperate button to handle button mashing for fateshift.
;*   *Note: this feature isn't implemented yet. Hotkeys for this game is
;*          a giant pain in the butt, due to the weird way it handles input.
;*          I've spent only 30mins playin the game so far, and waaaay too much
;*          time on these scripts
;*
;* Standard key remapping won't cut it in this game as, at least for the
;* mouse movements, the game chooses to read the input RAW (outside of ahk's
;* control) and so ahk can't prevent the game from seeing these presses. So
;* if you do a standard remap, you will get the new remapped action PLUS the
;* old "hard coded" map. Eg. Spells for right mouse button.   This is presumably
;* due to the action combat nature of the game, they wanted low latency (quick)
;* input in order for the combat to feel good/precise.
;*
;* So the remapping has to get a bit tricky. While we can't prevent the game
;* from seeing the mouse button presses, we can try to quickly "cancel" them
;* in the hopes the game will ignore them.  For the most part it seems to work
;* ok, although the first few presses upon loading the game seem to sneak through,
;* accurracy seems to be around at least 99% or so. Which is about the best we
;* can hope for. We do the cancel by quickly telling the game that we "let go" of
;* the button (even though we havent). If this is done fast enough, the game seems
;* to ignore the fact that the button was pressed at all, and so doesn't trigger
;* the in-game action (eg. cast a spell).
;*
;* Of course, to make things even better, the games menu system does not follow
; * the same raw reading of input. Meaning that the "Tricks" we use for combat
; * end up causing a bit of an issue with the menu ui. Basically left clicks in
; * the menu will act as if you are still holding down the click. Requiring two
; * clicks instead of one. To alleviate this, turn off the script using the toggle
; * key (Default "Home", configurable in the variables section). And turn back on
; * when you go back into the game.  Or you can use right click if you are lazy
; * which will simulate a normal left click in all the menus. (But then your primar/secondary
; * might be swapped when back in game, and require a reset)
;*
;* Special thanks to this technique, as posted on the BF3 Alpha Battlelog forums,
;* as that is where it was found. (After way too long spent google searching ;)
;*/



;/* The Name of the "ahk_class" of the game window. This is used so the
; * script will only be active when inside the game application.*/
GameAHKClass := "BigHugeClass" ;// ahk_class of Amalur game window


;/* VARIABLES - You can change the values of these (if you know what
; *             you are doing) to the keys you want to use/remap. Note
; *             in order for them to work the keys on the right side of
; *             the ":=" should be mapped in the game's control menu
; */
InGameToggleWeaponsKey := "q" ;// The key set to toggle weapons in the ingame controls menu
SetPrimaryWeaponButton := "MButton" ;// Button we will reuse to set our Left Click weapon
XButton1NewKey := "LShift" ;// The remapped key our side mouse button will use
XButton2NewKey := "RButton" ;// The remapped key our other side mouse button will use

HotkeysToggleButton := "Home" ;// The key that turns our hotkey on/off (toggles)


SecondaryWeaponSelected := false  ;// Do we have 1st or 2nd weapon selected?

WeaponChanged := ""

;Define Hotkeys so that they are only Active while in the game
; Note: This game is very sensitive about the type of Input sent. As noted
;       above, it will recieve all input events, regardless of whether or
;       not they are caught my a hotkey.  In addition, keypresses using SendPlay
;       method will be ignored totally, (Must use SentInput or SendEvent) and
;       even those will only regiser if held down for a certain duration (seems
;       to be greater then 20ms or so).
;=======================================================
;=======================================================
Hotkey, IfWinActive, ahk_class %GameAHKClass% ;// Script only works in Amalur game    

    Hotkey, %HotkeysToggleButton%, ToggleHotkeys
    Hotkey, %InGameToggleWeaponsKey%, ToggleWeapons,B P10
    Hotkey, %SetPrimaryWeaponButton%, SetPrimaryWeapon
  
return



;/* The ability to turn the hotkeys on/off.  Will be necessary as we are
; * rebinding the right mouse button, which we might not want for every part
; * of the game, especially if it has other uses. Also just good for safety to turn
; * off our hotkeys in case of any unforseen issues */
ToggleHotkeys:
    Suspend
    return



;/* All the below hotkeys will only work while in the Amalur game */
#IfWinActive ahk_class BigHugeClass


;/* If for some reason you decide to change weapons in the game (hit the
; * toggle weapons key). The script needs to know about it, to keep track
; * of the currently selected weapon. Hopefully you won't do this a lot
; * as it introduces the potential for going out to sync due to the ticky
; * timing of the fast paced game*/
ToggleWeapons:

    SendInput {Blind}{%InGameToggleWeaponsKey% Up} ;// Cancel original keypress    
    
    SendEvent {Blind}{%InGameToggleWeaponsKey%}  ;// Switch Weapons
    
    SecondaryWeaponSelected := !SecondaryWeaponSelected ;// Remember that we switched
    
    return


;/* Since this script doesn't work 100% of the time, we may need the ability to
; * easily set/reset the primary weapon. Ie. if the mouse buttons get switched.
; * When this key is pressed, the script will assume the selected weapon is the
; * primary weapon, and will assign it to the Left Mouse button.
; *
; * By Default this is the Middle mouse button (Scroll wheel button) since it
; * also changes weapons in the game. So you can fix it all by scrolling
; * then clicking the wheel. (Fast, and don't have to take hand off mouse)
SetPrimaryWeapon:
    SecondaryWeaponSelected:= 0
    return


;/* Experimental - Integrating L&R Mouse Buttons for Primary and Secondary Attack
; * 
; * Capture the Attack Key (left mouse button). By default this key will attack
; * with whatever the currently selected (in game) weapon is.  Our modified behavior
; * is that it will act as the "Primary Attack Key" (in conjunction with a seperate
; * secondary attack hotkey), and will only attack via the Primary weapon, regardless
; * of what weapon is selected in game.
; *
; * Logic: -since there is no discrete secondary attack key for Mouse/Keyboard controls
; *         (only gamepad), we have to hack/"fake" this functionality in, by using the
; *         in-game weapon toggle.  This is not foolproof though, and is rather ugly, as
; *         the script can never actually know what weapon is selected, it can only guess.
; *         Due to the very twitchy timing sentive nature of the gameplay (and how the game
; *         handles input, this method will not work %100 of the time.
; *       - we simply switch to the primary weapon (if it's not already selected) and send
; *         the attack command to the game.  This logic should be duplicated in another hotkey
; *         for the seconday attack functionality.
*$LButton::
    
    SendInput {LButton Up} ;// Cancel
    
    ;/* Switch to Primary weapon if it's not already selected */
    if (SecondaryWeaponSelected) {
        
      ;/* Re-use our ToggleWeapons code as that is what we want to do right now */
      GOSUB ToggleWeapons
    }       
    
    ;/* Now that we have our primary Weapon Selected, send the attack command to game */
    SendEvent {LButton Down}

    

    ;/* Tricky Part: This game uses charge attacks. So it's not just clicks, but long holds
    ; * that can work. So we have to be careful not to break the user's long hold press.
    ; * In a perfect world we would not send a matching LButton Up input then, and just let
    ; * the actual user's "release" go to the game. But we might have to handle the scenario that
    ; * the user clicked so fast that they released the button before we sent our own Press, and
    ; * so that press must be released.
    ; *
    ; * - We will first see if this timing issue ever comes up. If it does then we can try a simple
    ; *   keywait. If that doesn't give us enough resolution, then we may have to do a synchronized
    ; *   wait on another hotkey (yikes!)
    ; * - So far after (minimal) testing, charge attacks seem to work fine, and we aren't missing
    ; *   many (any?) regular clicks. So hopefully we won't need the below functionality. */
    
    ;/* Don't need this code as of yet, so commented out */
    ;keywait,LButton
    ;SendInput {LButton Up}
    
   return

;*$LButton Up::
    
    ;SendInput {LButton Up}
    ;return
 
;/* Experimental - Integrating L&R Mouse Buttons for Primary and Secondary Attack
; * 
; * Capture the Secondary attack Key (right mouse button). By default this key is hard mapped
; * to use ability. We can't actually stop this command from being sent upon click. What we can
; * do is try to quickly cancel out of it (send a click-release) so the game will ignore it.
; * Our modified behaviorthat it will act as the "Secondary Attack Key" (in conjunction with a seperate
; * primary attack hotkey), and will only attack via the Secondary weapon, regardless
; * of what weapon is selected in game.
; *
; * Logic: -since there is no discrete secondary attack key for Mouse/Keyboard controls
; *         (only gamepad), we have to hack/"fake" this functionality in, by using the
; *         in-game weapon toggle.  This is not foolproof though, and is rather ugly, as
; *         the script can never actually know what weapon is selected, it can only guess.
; *         Due to the very twitchy timing sentive nature of the gameplay (and how the game
; *         handles input, this method will not work %100 of the time.
; *       - we simply switch to the secondary weapon (if it's not already selected) and send
; *         the attack command to the game.  This logic should be duplicated in another hotkey
; *         for the primary attack functionality.
*$RButton::       
   
    SendInput {RButton Up} ;// Cancel the ability use
    
    ;/* Switch to Secondary weapon if it's not already selected */
    if (!SecondaryWeaponSelected) {
        
      ;/* Re-use our ToggleWeapons code as that is what we want to do right now */
      GOSUB ToggleWeapons
    }       
    
    ;/* Now that we have our primary Weapon Selected, send the attack command to game */
    SendInput {LButton Down}
    return




;/* Handle release of the RButton: In our Left Mouse button example, we don't
; * need this functionality, as the user will send this event when they let go. However we
; * are translating the RButton into an LButton press, and so they will be letting go of the
; * RButton when the game needs to see them letting go of the LButton. So we must do this
; * translation ourself.
; *
; *
; */
*$RButton Up::

    SetMouseDelay 1
    
    SendInput {Blind}{LButton Up}
    return

;/* Side Buttons aren't mapped at all in game, so we don't need anything fancy. Just
; * a simple remap will do.  Since we are using configurable remap keys (variables)
; * we need to write the full definition (both down and up hotkeys) instead of the
; * simple mapping of "a::b". */
*XButton1::

    
    
    SendInput {Blind}{%XButton1NewKey% DownTemp}
    return

*XButton1 up::

    
    SendInput {Blind}{%XButton1NewKey% Up}
    return



;/* Side Buttons aren't mapped at all in game, so we don't need anything fancy. Just
; * a simple remap will do.  Since we are using configurable remap keys (variables)
; * we need to write the full definition (both down and up hotkeys) instead of the
; * simple mapping of "a::b". */
*XButton2::

      
    SendInput {Blind}{%XButton2NewKey% DownTemp}
    return

*XButton2 up::

    
    SendInput {Blind}{%XButton2NewKey% Up}
    return





;/* Experimental - Added support for ability select, to become "use hotkeys"
; *
; * Personally this isn't something I'd use, but it seems people would like it.
; * Rather then selecting abilities with the number keys and then having to
; * press the use ability button, it'd be nice to just press the ability hotkey
; * to select and use all at once. Kinda how it works in many MMOs?
; *
; * This shouldn't be too tricky, unless it messes with our Dual Attack Key setup
; * above.
; *
; * Spoke too soon, this is super tricky, timingn again is a wierd issue. Simply
; * remapping the key should do it, but triggers it super inconsitently.
; *
; * Ended up having to do a bit of waiting using the Sleep Command, for the magic
; * 22ms amount. What a strange game.
; */
*~1::
*~2::
*~3::
*~4::
*~5::
*~6::
*~7::
*~8::
*~9::
*~0::    
    Sleep 22
    SendInput {RButton Down}
    Sleep 22
    SendInput {RButton Up}
    return




;/* Since timing is so important, the priority of the threads of each individual
; * remap/hotkey becomes an issue. We need to cancel our Clicks as soon as possible
; * Right click (ability) even more so.
; *
; * Priority in order of who should execute first:
; *    Right Click Down, Left Click Down, Toggle Weapons, Side Buttons */
Hotkey, *RButton,,P0
Hotkey, *$LButton,,P5
Hotkey, *XButton1,,P15
Hotkey, *XButton2,,P15
