;#######################################################
#NoEnv
#InstallKeybdHook
#InstallMouseHook 
SendMode Input
SetTitleMatchMode, RegEx

;/* Timing is very important in this game. Want our presses to be as fast as
; * possible, and held down just long enough for the game to register them */
SetKeyDelay, -1, 22
SetMouseDelay 1


;/*****************************************************************************
;* Autohotkey Script for Kingdoms of Amalur Reckoning - Regular v1.0
;*
;* - Primary purpose to to allow rebinding of the right mouse button, and
;*   optional additional side mouse buttons (up to 2).
;*
;* - Additional feature of turning the ability select keys (0-9) into "use hotkeys"
;*   ie. the ability will fire when you select them.
;*
;* Standard key remapping won't cut it in this game as, at least for the
;* mouse movements, the game chooses to read the input RAW (outside of ahk's
;* control) and so ahk can't prevent the game from seeing these presses. So
;* if you do a standard remap, you will get the new remapped action PLUS the
;* old "hard coded" map. Eg. Spells for right mouse button.   This is presumably
;* due to the action combat nature of the game, they wanted low latency (quick)
;* input in order for the combat to feel good/precise.
;*
;* So the remapping has to get a bit tricky. While we can't prevent the game
;* from seeing the mouse button presses, we can try to quickly "cancel" them
;* in the hopes the game will ignore them.  For the most part it seems to work
;* ok, although the first few presses upon loading the game seem to sneak through,
;* accurracy seems to be around at least 99% or so. Which is about the best we
;* can hope for. We do the cancel by quickly telling the game that we "let go" of
;* the button (even though we havent). If this is done fast enough, the game seems
;* to ignore the fact that the button was pressed at all, and so doesn't trigger
;* the in-game action (eg. cast a spell).
;*
;* Special thanks to this technique, as posted on the BF3 Alpha Battlelog forums,
;* as that is where it was found. (After way too long spent google searching ;)
;*/



;/* The Name of the "ahk_class" of the game window. This is used so the
; * script will only be active when inside the game application.*/
GameAHKClass := "BigHugeClass" ;// ahk_class of Amalur game window


;/* VARIABLES - You can change the values of these (if you know what
; *             you are doing) to the keys you want to use/remap. Note
; *             in order for them to work the keys on the right side of
; *             the ":=" should be mapped in the game's control menu
; */
RButtonNewKey := "w" ;// The remapped key our Right Mouse button press will be translated to
XButton1NewKey := "l" ;// The remapped key our side mouse button will use
XButton2NewKey := "RButton" ;// The remapped key our other side mouse button will use
HotkeysToggleButton := "Home" ;// The key that turns our hotkey on/off (toggles)




;Define Hotkeys so that they are only Active while in the game
;=======================================================
;=======================================================
Hotkey, IfWinActive, ahk_class %GameAHKClass% ;// Script only works in Amalur game    

    Hotkey, %HotkeysToggleButton%, ToggleHotkeys
  
return


;/* The ability to turn the hotkeys on/off.  Will be necessary as we are
; * rebinding the right mouse button, which we might not want for every part
; * of the game, especially if it has other uses. Also just good for safety to turn
; * off our hotkeys in case of any unforseen issues */
ToggleHotkeys:
    Suspend
    return



;/* All the below hotkeys will only work while in the Amalur game */
#IfWinActive ahk_class BigHugeClass



;/* While we can't stop the game from seeing our mouse pressed, we can at
; * least try our best to "cancel" them.  So we try to quickly send the "release"
; * of the button as soon as it's pressed.  That way the game "ignores" the
; * press and doesn't execute the ingame action bound to it. Most of the time
; * this works. After we have released the button, we can then send the standard
; * key remapping */
*RButton::
    
    ;/* Quickly send the game that we have release the button. If we do it fast enough it won't
    ; * trigger the in game command bound to the button. */    
    SendInput {Blind}{RButton Up}
    
    ;/* Quickly send our remapped key to the game. There is a slighty latency as we had to
    ; * send the above "cancelation" command first, but that one has to take priority */    
    SendInput {Blind}{%RButtonNewKey% down}
    
    return

;/* Borring part, standard hotkey key remapping: When we are done pressing
; * the button, we must also "release" the remapped key. */
*RButton Up::
    
    Send {Blind}{%RButtonNewKey% up}
    
    return  


;/* Side Buttons aren't mapped at all in game, so we don't need anything fancy. Just
; * a simple remap will do.  Since we are using configurable remap keys (variables)
; * we need to write the full definition (both down and up hotkeys) instead of the
; * simple mapping of "a::b". */
*XButton1::
     
    Send {Blind}{%XButton1NewKey% DownTemp}
    return


*XButton1 Up::
   
    Send {Blind}{%XButton1NewKey% Up}
    return



;/* Side Buttons aren't mapped at all in game, so we don't need anything fancy. Just
; * a simple remap will do.  Since we are using configurable remap keys (variables)
; * we need to write the full definition (both down and up hotkeys) instead of the
; * simple mapping of "a::b". */
*XButton2::
      
    Send {Blind}{%XButton2NewKey% DownTemp}
    return


*XButton2 Up::

   
    Send {Blind}{%XButton2NewKey% Up}
    return



;/* Experimental - Added support for ability select, to become "use hotkeys"
; *
; * Personally this isn't something I'd use, but it seems people would like it.
; * Rather then selecting abilities with the number keys and then having to
; * press the use ability button, it'd be nice to just press the ability hotkey
; * to select and use all at once. Kinda how it works in many MMOs?
; *
; * This shouldn't be too tricky, unless it messes with our Dual Attack Key setup
; * above.
; *
; * Spoke too soon, this is super tricky, timingn again is a wierd issue. Simply
; * remapping the key should do it, but triggers it super inconsitently.
; *
; * Ended up having to do a bit of waiting using the Sleep Command, for the magic
; * 22ms amount. What a strange game.
; */
*~1::
*~2::
*~3::
*~4::
*~5::
*~6::
*~7::
*~8::
*~9::
*~0::    
    Sleep 22
    SendInput {RButton Down}
    Sleep 22
    SendInput {RButton Up}
    return


